@@ -45,7 +45,7 @@ class MainConfig:
     DBFilePath = str(r'/root/FTSDataBase.db')
 
     # the version information of the server (recommended to leave as default)
-    version = 'FreeTAKServer-1.5 RC 1'
+    version = 'FreeTAKServer-1.5 RC 2'
 
     MainPath = str(Path(fr'{userpath}{python_version}/dist-packages/FreeTAKServer'))
 
@@ -79,11 +79,11 @@ class MainConfig:
 
     federationCert = str(Path(fr'{MainPath}/certs/pubserver.pem'))
     federationKey = str(Path(fr'{MainPath}/certs/pubserver.key'))
-    federationKeyPassword = str('your pass')
+    federationKeyPassword = str('defaultpass')
     
     # location to backup client packages
     clientPackages = str(Path(fr'{MainPath}/certs/ClientPackages'))
 
-    password = str('your pass')
+    password = str('defaultpass')
 
     websocketkey = "YourWebsocketKey"
\ No newline at end of file
