@@ -15,9 +15,11 @@ from FreeTAKServer.controllers.AddDataToCoTList import AddDataToCoTList
 from FreeTAKServer.model.FilterGroup import FilterGroup
 from FreeTAKServer.controllers.services.SSLCoTServiceController import SSLCoTServiceController
 from FreeTAKServer.controllers.services.TCPCoTServiceController import TCPCoTServiceController
-from FreeTAKServer.controllers.services.FederationClientServiceController import FederationClientServiceController
+from FreeTAKServer.controllers.services.federation.FederationClientService import FederationClientServiceController
+from FreeTAKServer.controllers.services.federation.federation import FederationServerService
 from FreeTAKServer.controllers.services.FederationServerServiceController import FederationServerServiceController
 from FreeTAKServer.controllers.DatabaseControllers.DatabaseController import DatabaseController
+from FreeTAKServer.controllers.certificate_generation import AtakOfTheCerts
 
 loggingConstants = LoggingConstants()
 logger = CreateLoggerController("FTS").getLogger()
@@ -50,7 +52,7 @@ class FTS:
             self.receive_Rest_stopper = threading.Event()
             self.receive_Rest_stopper.clear()
             self.receive_Rest = threading.Thread(target=self.receive_Rest_commands, args=(self.receive_Rest_stopper,))
-            self.RestAPIProcess = multiprocessing.Process(target=RestAPI().startup, args=(RestAPIPipe, self.RestAPICommands, StartupObjects.RestAPIService.RestAPIServiceIP, StartupObjects.RestAPIService.RestAPIServicePort))
+            self.RestAPIProcess = multiprocessing.Process(target=RestAPI().startup, args=(RestAPIPipe, self.RestAPICommands, StartupObjects.RestAPIService.RestAPIServiceIP, StartupObjects.RestAPIService.RestAPIServicePort, self.StartupTime))
             self.receive_Rest.start()
             self.RestAPIProcess.start()
             self.pipeList['restAPI'] = self.RestAPIPipe
@@ -92,7 +94,7 @@ class FTS:
             self.ClientDataPipe.close()
             self.CoTPoisonPill.clear()
 
-            time.sleep(0.1)
+            time.sleep(1)
             if self.CoTService.is_alive():
                 self.CoTService.terminate()
                 self.CoTService.join()
@@ -124,6 +126,8 @@ class FTS:
             return -1
 
     def stop_tcp_data_package_service(self):
+        del self.pipeList['tcp_data_package_service_pipe']
+        self.FilterGroup.sources.remove(self.tcp_data_package_service_pipe)
         try:
             self.TCPDataPackageService.terminate()
         except Exception as e:
@@ -134,6 +138,7 @@ class FTS:
         except Exception as e:
             logger.error("there's been an exception in the joining of DataPackage Service " + str(e))
             return -1
+        self.tcp_data_package_service_pipe.close()
         return 1
 
     def start_ssl_data_package_service(self, FTSServiceStartupConfigObject):
@@ -153,6 +158,8 @@ class FTS:
             return -1
 
     def stop_ssl_data_package_service(self):
+        del(self.pipeList['ssl_data_package_service'])
+        self.FilterGroup.sources.remove(self.ssl_data_package_service)
         try:
             self.SSLDataPackageService.terminate()
         except Exception as e:
@@ -163,6 +170,7 @@ class FTS:
         except Exception as e:
             logger.error("there's been an exception in the joining of DataPackage Service " + str(e))
             return -1
+        self.ssl_data_package_service.close()
         return 1
 
     def start_SSL_CoT_service(self, FTSServiceStartupConfigObject):
@@ -189,14 +197,14 @@ class FTS:
             self.SSLCoTPoisonPill.clear()
 
             time.sleep(0.1)
-            if self.CoTService.is_alive():
-                self.CoTService.terminate()
-                self.CoTService.join()
+            if self.SSLCoTService.is_alive():
+                self.SSLCoTService.terminate()
+                self.SSLCoTService.join()
             else:
-                self.CoTService.join()
-
-            self.FilterGroup.sources.remove(self.TCPCoTServiceFTSPipe)
-            self.FilterGroup.receivers.remove(self.TCPCoTServiceFTSPipe)
+                self.SSLCoTService.join()
+            del(self.pipeList["SSLCoTServiceFTSPipe"])
+            self.FilterGroup.sources.remove(self.SSLCoTServicePipe)
+            self.FilterGroup.receivers.remove(self.SSLCoTServicePipe)
 
         except Exception as e:
             logger.error("there's been an exception in the stopping of CoT Service " + str(e))
@@ -204,22 +212,19 @@ class FTS:
         return 1
 
     def start_federation_client_service(self, FTSServiceStartupConfigObject):
-        try:
-            ip = FTSServiceStartupConfigObject.FederationClientService.FederationClientServiceIP
-            port = FTSServiceStartupConfigObject.FederationClientService.FederationClientServicePort
-            FederationClientServicePipe, self.FederationClientServicePipeFTS = multiprocessing.Pipe()
-            self.FederationClientService = multiprocessing.Process(
-                target=FederationClientServiceController(FederationClientServicePipe, ip, port).start, args=())
-            self.FederationClientService.start()
-            self.pipeList['FederationClientServiceFTSPipe'] = self.FederationClientServicePipeFTS
-            self.FilterGroup.sources.append(self.FederationClientServicePipeFTS)
-            self.FilterGroup.receivers.append(self.FederationClientServicePipeFTS)
-            return 1
-        except:
-            return -1
+        FederationClientServicePipe, self.FederationClientServicePipeFTS = multiprocessing.Pipe(True)
+        self.FederationClientService = multiprocessing.Process(target=FederationClientServiceController().start, args=(FederationClientServicePipe,))
+        self.FederationClientService.start()
+        self.pipeList['FederationClientServiceFTSPipe'] = self.FederationClientServicePipeFTS
+        self.FilterGroup.sources.append(self.FederationClientServicePipeFTS)
+        self.FilterGroup.receivers.append(self.FederationClientServicePipeFTS)
+        return 1
 
     def stop_federation_client_service(self):
         try:
+            del(self.pipeList['FederationClientServiceFTSPipe'])
+            self.FilterGroup.receivers.remove(self.FederationClientServicePipeFTS)
+            self.FilterGroup.sources.remove(self.FederationClientServicePipeFTS)
             if self.FederationClientService.is_alive():
                 self.FederationClientService.terminate()
                 self.FederationClientService.join()
@@ -238,7 +243,7 @@ class FTS:
                 port = FTSServiceStartupConfigObject.FederationServerService.FederationServerServicePort
                 FederationServerServicePipe, self.FederationServerServicePipeFTS = multiprocessing.Pipe()
                 self.FederationServerService = multiprocessing.Process(
-                    target=FederationServerServiceController(FederationServerServicePipe, ip, port).start, args=())
+                    target=FederationServerService().start, args=(FederationServerServicePipe, ip, port))
                 self.FederationServerService.start()
                 self.pipeList['FederationServerServiceFTSPipe'] = self.FederationServerServicePipeFTS
                 self.FilterGroup.sources.append(self.FederationServerServicePipeFTS)
@@ -254,7 +259,7 @@ class FTS:
                 self.FederationServerService.join()
             else:
                 self.FederationServerService.join()
-
+            self.FederationServerServicePipeFTS.close()
             self.FilterGroup.sources.remove(self.FederationServerServicePipeFTS)
             self.FilterGroup.receivers.remove(self.FederationServerServicePipeFTS)
             return 1
@@ -264,53 +269,89 @@ class FTS:
     #change object name to FTSServiceStartupConfigObject
     def start_all(self, FTSServiceStartupConfigObject):
         try:
-            self.FTSServiceStartupConfigObject = FTSServiceStartupConfigObject
             if FTSServiceStartupConfigObject.TCPDataPackageService.TCPDataPackageServiceStatus == 'start':
+                self.FTSServiceStartupConfigObject.TCPDataPackageService.TCPDataPackageServiceStatus = FTSServiceStartupConfigObject.TCPDataPackageService.TCPDataPackageServiceStatus
                 self.start_tcp_data_package_service(FTSServiceStartupConfigObject)
-
+                if FTSServiceStartupConfigObject.TCPDataPackageService.TCPDataPackageServicePort != "":
+                    self.FTSServiceStartupConfigObject.TCPDataPackageService.TCPDataPackageServicePort = FTSServiceStartupConfigObject.TCPDataPackageService.TCPDataPackageServicePort
+                if FTSServiceStartupConfigObject.TCPDataPackageService.TCPDataPackageServiceIP != "":
+                    self.FTSServiceStartupConfigObject.TCPDataPackageService.TCPDataPackageServiceIP = FTSServiceStartupConfigObject.TCPDataPackageService.TCPDataPackageServiceIP
+                
             elif FTSServiceStartupConfigObject.TCPDataPackageService.TCPDataPackageServiceStatus == 'stop':
+                self.FTSServiceStartupConfigObject.TCPDataPackageService.TCPDataPackageServiceStatus = FTSServiceStartupConfigObject.TCPDataPackageService.TCPDataPackageServiceStatus
                 self.stop_tcp_data_package_service()
             else:
                 pass
 
             if FTSServiceStartupConfigObject.SSLDataPackageService.SSLDataPackageServiceStatus == 'start':
+                self.FTSServiceStartupConfigObject.SSLDataPackageService.SSLDataPackageServiceStatus = FTSServiceStartupConfigObject.SSLDataPackageService.SSLDataPackageServiceStatus
                 self.start_ssl_data_package_service(FTSServiceStartupConfigObject)
-
+                if FTSServiceStartupConfigObject.SSLDataPackageService.SSLDataPackageServicePort != "":
+                    self.FTSServiceStartupConfigObject.SSLDataPackageService.SSLDataPackageServicePort = FTSServiceStartupConfigObject.SSLDataPackageService.SSLDataPackageServicePort
+                if FTSServiceStartupConfigObject.SSLDataPackageService.SSLDataPackageServiceIP != "":
+                    self.FTSServiceStartupConfigObject.SSLDataPackageService.SSLDataPackageServiceIP = FTSServiceStartupConfigObject.SSLDataPackageService.SSLDataPackageServiceIP
             elif FTSServiceStartupConfigObject.SSLDataPackageService.SSLDataPackageServiceStatus == 'stop':
+                self.FTSServiceStartupConfigObject.SSLDataPackageService.SSLDataPackageServiceStatus = FTSServiceStartupConfigObject.SSLDataPackageService.SSLDataPackageServiceStatus
                 self.stop_ssl_data_package_service()
             else:
                 pass
 
             if FTSServiceStartupConfigObject.CoTService.CoTServiceStatus == 'start':
+                self.FTSServiceStartupConfigObject.CoTService.CoTServiceStatus = FTSServiceStartupConfigObject.CoTService.CoTServiceStatus
                 self.start_CoT_service(FTSServiceStartupConfigObject)
+                if FTSServiceStartupConfigObject.CoTService.CoTServicePort != "":
+                    self.FTSServiceStartupConfigObject.CoTService.CoTServicePort = FTSServiceStartupConfigObject.CoTService.CoTServicePort
+                if FTSServiceStartupConfigObject.CoTService.CoTServiceIP != "":
+                    self.FTSServiceStartupConfigObject.CoTService.CoTServiceIP = FTSServiceStartupConfigObject.CoTService.CoTServiceIP
 
             elif FTSServiceStartupConfigObject.CoTService.CoTServiceStatus == 'stop':
+                self.FTSServiceStartupConfigObject.CoTService.CoTServiceStatus = FTSServiceStartupConfigObject.CoTService.CoTServiceStatus
                 self.stop_CoT_service()
             else:
                 pass
 
             if FTSServiceStartupConfigObject.SSLCoTService.SSLCoTServiceStatus == 'start':
+                self.FTSServiceStartupConfigObject.SSLCoTService.SSLCoTServiceStatus = FTSServiceStartupConfigObject.SSLCoTService.SSLCoTServiceStatus
                 self.start_SSL_CoT_service(FTSServiceStartupConfigObject)
-
+                if FTSServiceStartupConfigObject.SSLCoTService.SSLCoTServicePort != "":
+                    self.FTSServiceStartupConfigObject.SSLCoTService.SSLCoTServicePort = FTSServiceStartupConfigObject.SSLCoTService.SSLCoTServicePort
+                if FTSServiceStartupConfigObject.SSLCoTService.SSLCoTServiceIP != "":
+                    self.FTSServiceStartupConfigObject.SSLCoTService.SSLCoTServiceIP = FTSServiceStartupConfigObject.SSLCoTService.SSLCoTServiceIP
+                    
             elif FTSServiceStartupConfigObject.SSLCoTService.SSLCoTServiceStatus == 'stop':
+                self.FTSServiceStartupConfigObject.SSLCoTService.SSLCoTServiceStatus = FTSServiceStartupConfigObject.SSLCoTService.SSLCoTServiceStatus
                 self.stop_SSL_CoT_service()
 
             else:
                 pass
 
             if FTSServiceStartupConfigObject.FederationClientService.FederationClientServiceStatus == 'start':
+                self.FTSServiceStartupConfigObject.FederationClientService.FederationClientServiceStatus = FTSServiceStartupConfigObject.FederationClientService.FederationClientServiceStatus
                 self.start_federation_client_service(FTSServiceStartupConfigObject)
+                if FTSServiceStartupConfigObject.FederationClientService.FederationClientServicePort != "":
+                    self.FTSServiceStartupConfigObject.FederationClientService.FederationClientServicePort = FTSServiceStartupConfigObject.FederationClientService.FederationClientServicePort
+                if FTSServiceStartupConfigObject.FederationClientService.FederationClientServiceIP != "":
+                    self.FTSServiceStartupConfigObject.FederationClientService.FederationClientServiceIP = FTSServiceStartupConfigObject.FederationClientService.FederationClientServiceIP
+
 
             elif FTSServiceStartupConfigObject.FederationClientService.FederationClientServiceStatus == 'stop':
+                self.FTSServiceStartupConfigObject.FederationClientService.FederationClientServiceStatus = FTSServiceStartupConfigObject.FederationClientService.FederationClientServiceStatus
                 self.stop_federation_client_service()
 
             else:
                 pass
 
             if FTSServiceStartupConfigObject.FederationServerService.FederationServerServiceStatus == 'start':
+                self.FTSServiceStartupConfigObject.FederationServerService.FederationServerServiceStatus = FTSServiceStartupConfigObject.FederationServerService.FederationServerServiceStatus
                 self.start_federation_server_service(FTSServiceStartupConfigObject)
+                if FTSServiceStartupConfigObject.FederationServerService.FederationServerServicePort != "":
+                    self.FTSServiceStartupConfigObject.FederationServerService.FederationServerServicePort = FTSServiceStartupConfigObject.FederationServerService.FederationServerServicePort
+                if FTSServiceStartupConfigObject.FederationServerService.FederationServerServiceIP != "":
+                    self.FTSServiceStartupConfigObject.FederationServerService.FederationServerServiceIP = FTSServiceStartupConfigObject.FederationServerService.FederationServerServiceIP
+
 
             elif FTSServiceStartupConfigObject.FederationServerService.FederationServerServiceStatus == 'stop':
+                self.FTSServiceStartupConfigObject.FederationServerService.FederationServerServiceStatus = FTSServiceStartupConfigObject.FederationServerService.FederationServerServiceStatus
                 self.stop_federation_server_service()
 
             else:
@@ -333,6 +374,7 @@ class FTS:
                 self.socketCount = data[2]
                 if data[0] == 'add':
                     clientArray.append(data[1])
+                    self.RestAPIPipe.send(data[1])
                 else:
                     for client in clientArray:
                         if client.ID == data[1].ID:
@@ -349,8 +391,7 @@ class FTS:
                                 pass
                     else:
                         pass
-                userData = self.show_users()
-                self.RestAPIPipe.send(userData)
+
                 return self.clientArray
             else:
                 return self.clientArray
@@ -361,8 +402,10 @@ class FTS:
         while kill.is_set() == False:
             try:
                 command = self.RestAPICommandsFTS.recv()
+                if isinstance(command, tuple):
+                    self.FederationClientServicePipeFTS.send(command)
                 function = getattr(self, command[0])
-                if len(command)>=2:
+                if len(command) >= 2:
                     output = function(command[1])
                 else:
                     output = function()
@@ -372,10 +415,6 @@ class FTS:
                 logger.error('there has been an exception thrown in processing rest command ' + str(e))
                 self.RestAPICommandsFTS.send('500')
 
-            except Exception as e:
-                logger.error('there has been a critical exception ' + str(e))
-                break
-
     def check_server_status(self):
         return self.FTSServiceStartupConfigObject
 
@@ -463,7 +502,7 @@ class FTS:
                     for client in self.clientArray:
                         AddDataToCoTList().send(self.FilterGroup.receivers, client)
                 # this runs in all other cases in which data is received
-                elif data != 0:
+                elif data != 0 and data is not None:
                     try:
                         print('data received in FTS ' + str(data.xmlString))
                     except:
@@ -475,12 +514,21 @@ class FTS:
         except Exception as e:
             pass
 
-    def startup(self, CoTPort, CoTIP, DataPackagePort, DataPackageIP, RestAPIPort, RestAPIIP, SSLCoTPort, SSLCoTIP, AutoStart):
+    def startup(self, CoTPort, CoTIP, DataPackagePort, DataPackageIP, SSLDataPackagePort, SSLDataPackageIP, RestAPIPort, RestAPIIP, SSLCoTPort, SSLCoTIP, AutoStart, firstStart = False):
         try:
+            self.FTSServiceStartupConfigObject.RestAPIService.RestAPIServiceStatus = 'start'
+            self.FTSServiceStartupConfigObject.RestAPIService.RestAPIServicePort = RestAPIPort
+            self.FTSServiceStartupConfigObject.RestAPIServiceIP = RestAPIIP
+            if firstStart:
+                from datetime import datetime as dt
+                self.StartupTime = dt.now()
+            else:
+                pass
             if AutoStart == 'False':
                 StartupObject = FTSObj()
                 StartupObject.RestAPIService.RestAPIServicePort = RestAPIPort
                 StartupObject.RestAPIService.RestAPIServiceIP = RestAPIIP
+                StartupObject.RestAPIService.RestAPIServiceStatus = 'start'
                 self.start_restAPI_service(StartupObject)
 
             else:
@@ -493,10 +541,18 @@ class FTS:
                 StartupObject.TCPDataPackageService.TCPDataPackageServicePort = DataPackagePort
                 StartupObject.TCPDataPackageService.TCPDataPackageServiceStatus = 'start'
 
+                StartupObject.SSLDataPackageService.SSLDataPackageServiceIP = SSLDataPackageIP
+                StartupObject.SSLDataPackageService.SSLDataPackageServicePort = SSLDataPackagePort
+                StartupObject.SSLDataPackageService.SSLDataPackageServiceStatus = 'start'
+
                 StartupObject.RestAPIService.RestAPIServicePort = RestAPIPort
                 StartupObject.RestAPIService.RestAPIServiceIP = RestAPIIP
                 StartupObject.RestAPIService.RestAPIServiceStatus = 'start'
 
+                StartupObject.FederationClientService.FederationClientServiceStatus = 'start'
+
+                StartupObject.FederationServerService.FederationServerServiceStatus = 'start'
+
                 StartupObject.SSLCoTService.SSLCoTServiceStatus = 'start'
                 StartupObject.SSLCoTService.SSLCoTServiceIP = SSLCoTIP
                 StartupObject.SSLCoTService.SSLCoTServicePort = SSLCoTPort
@@ -528,13 +584,18 @@ if __name__ == "__main__":
                             default=FTSObj().TCPDataPackageService.TCPDataPackageServicePort)
         parser.add_argument('-DataPackageIP', type=str, help=OrchestratorConstants().APIPORTDESC,
                             default=FTSObj().TCPDataPackageService.TCPDataPackageServiceIP)
+        parser.add_argument('-SSLDataPackagePort', type=int, help=OrchestratorConstants().APIPORTDESC,
+                            default=FTSObj().SSLDataPackageService.SSLDataPackageServicePort)
+        parser.add_argument('-SSLDataPackageIP', type=str, help=OrchestratorConstants().APIPORTDESC,
+                            default=FTSObj().SSLDataPackageService.SSLDataPackageServiceIP)
         parser.add_argument('-RestAPIPort', type=int, help=OrchestratorConstants().APIPORTDESC,
                             default=FTSObj().RestAPIService.RestAPIServicePort)
         parser.add_argument('-RestAPIIP', type=str, help=OrchestratorConstants().APIPORTDESC,
                             default=FTSObj().RestAPIService.RestAPIServiceIP)
         parser.add_argument('-AutoStart', type=str, help='wether or not you want all services to start or only the root service and the RestAPI service', default='True')
         args = parser.parse_args()
+        AtakOfTheCerts().bake_startup()
         CreateStartupFilesController()
-        FTS().startup(args.CoTPort, args.CoTIP, args.DataPackagePort, args.DataPackageIP, args.RestAPIPort, args.RestAPIIP, args.SSLCoTPort, args.SSLCoTIP, args.AutoStart)
+        FTS().startup(args.CoTPort, args.CoTIP, args.DataPackagePort, args.DataPackageIP, args.SSLDataPackagePort, args.SSLDataPackageIP, args.RestAPIPort, args.RestAPIIP, args.SSLCoTPort, args.SSLCoTIP, args.AutoStart, True)
     except Exception as e:
         print(e)
\ No newline at end of file
