@@ -192,16 +192,19 @@ class FederationServerService(ServerServiceInterface, ServiceBase):
     def send_data_to_clients(self, data):
         from lxml import etree
         try:
-            xmlstring = data.xmlString
-            detail = etree.fromstring(xmlstring).find('detail')
-            protobuf = ProtobufSerializer().from_fts_object_to_format(data.modelObject)
-            protobuf.event.other = etree.tostring(detail)
-            protobufstring = protobuf.SerializeToString()
-            header = self._generate_header(len(protobufstring))
-            protobufstring = header + protobufstring
-            print(protobufstring)
-            for client in self.federates:
-                client.conn.send(protobufstring)
+            if self.federates:
+                xmlstring = data.xmlString
+                detail = etree.fromstring(xmlstring).find('detail')
+                protobuf = ProtobufSerializer().from_fts_object_to_format(data.modelObject)
+                protobuf.event.other = etree.tostring(detail)
+                protobufstring = protobuf.SerializeToString()
+                header = self._generate_header(len(protobufstring))
+                protobufstring = header + protobufstring
+                print(protobufstring)
+                for client in self.federates:
+                    client.conn.send(protobufstring)
+            else:
+                return None
         except Exception as e:
             pass
 
@@ -213,24 +216,29 @@ class FederationServerService(ServerServiceInterface, ServiceBase):
         return None
 
     def send_connection_data(self, CoT: ClientInformation):
-        proto_obj = FederatedEvent()
-        proto_obj.contact.uid = str(CoT.modelObject.uid)
-        proto_obj.contact.callsign = str(CoT.modelObject.detail.contact.callsign)
-        proto_obj.contact.operation = 1
-        proto_str = proto_obj.SerializeToString()
-        header = self._generate_header(len(proto_str))
-        for fed in self.federates:
-            fed.conn.send(header + proto_str)
-
+        if self.federates:
+            proto_obj = FederatedEvent()
+            proto_obj.contact.uid = str(CoT.modelObject.uid)
+            proto_obj.contact.callsign = str(CoT.modelObject.detail.contact.callsign)
+            proto_obj.contact.operation = 1
+            proto_str = proto_obj.SerializeToString()
+            header = self._generate_header(len(proto_str))
+            for fed in self.federates:
+                fed.conn.send(header + proto_str)
+        else:
+            return None
     def send_disconnection_data(self, CoT: SendDisconnect):
-        proto_obj = FederatedEvent()
-        proto_obj.contact.uid = str(CoT.modelObject.detail.link.uid)
-        proto_obj.contact.callsign = str(CoT.modelObject.detail.link.type)
-        proto_obj.contact.operation = 4
-        proto_str = proto_obj.SerializeToString()
-        header = self._generate_header(len(proto_str))
-        for fed in self.federates:
-            fed.conn.send(header + proto_str)
+        if self.federates:
+            proto_obj = FederatedEvent()
+            proto_obj.contact.uid = str(CoT.modelObject.detail.link.uid)
+            proto_obj.contact.callsign = str(CoT.modelObject.detail.link.type)
+            proto_obj.contact.operation = 4
+            proto_str = proto_obj.SerializeToString()
+            header = self._generate_header(len(proto_str))
+            for fed in self.federates:
+                fed.conn.send(header + proto_str)
+        else:
+            return None
 
     def start(self, pipe, ip, port):
         self.db = DatabaseController()
